

__version__ = '0.9.2'
__copyright__ = 'Copyright (c) 2016 Kevin Walchko'
__license__ = 'MIT'
__author__ = 'Kevin J. Walchko'


from .ServoSerial import ServoSerial
# from .DummySerial import DummySerial
from . import utils
# from utils import listSerialPorts
from . import xl320
from . import Packet
